#include <J3ML/Geometry/Rect2D.hpp>

namespace J3ML::Geometry
{
    Rect2D Rect2D::operator+(const J3ML::LinearAlgebra::Vector2 &pt) const {
        return {position+pt, size};
    }

    Rect2D &Rect2D::operator+(const Vector2 &pt) {
        position += pt;
        return *this;
    }

    AABB2D Rect2D::GetAsAABB() const { return {MinPoint(), MaxPoint()};}

    Rect2D Rect2D::FromCentroidAndRadii(const Vector2 &centroid, const Vector2 &radii) {
        return Rect2D(centroid.x - (radii.x), centroid.y - (radii.y),
                      radii.x*2.f, radii.y*2.f);
    }

    Rect2D::Rect2D(float x, float y, float w, float h) {
        this->position = {x,y};
        this->size = {w,h};
    }

    Rect2D::Rect2D(const Vector2 &pos, const Vector2 &size) {
        this->position = pos;
        this->size = size;
    }

    float Rect2D::HorizontalRadius() const { return Width()/2.f;}

    float Rect2D::VerticalRadius() const { return Height()/2.f;}

    float Rect2D::HalfWidth() const { return HorizontalRadius();}

    float Rect2D::HalfHeight() const { return VerticalRadius();}

    Vector2 Rect2D::Centroid() const { return position + (size / 2.f);}

    float Rect2D::Width() const { return size.x;}

    float Rect2D::Height() const { return size.y;}

    Vector2 Rect2D::MinPoint() const { return position; }

    Vector2 Rect2D::MaxPoint() const { return position + size;}


}